@@ -24,13 +24,23 @@
  */
 #include "Host/None/ImageDialog.h"
 #include <QDebug>
+#include <QFileInfo>
+#include <QImageWriter>
+#include <QMessageBox>
+#include <QString>
+#include <QStringList>
+#include "Common.h"
+#include "JpegQualityDialog.h"
 #include "gmic.h"
 
+namespace gmic_qt_standalone
+{
+
 ImageView::ImageView(QWidget * parent) : QWidget(parent) {}
 
 void ImageView::setImage(const cimg_library::CImg<gmic_pixel_type> & image)
 {
-  ImageConverter::convert(image, _image);
+  GmicQt::convertCImgToQImage(image, _image);
   setMinimumSize(std::min(640, image.width()), std::min(480, image.height()));
 }
 
@@ -40,13 +50,26 @@ void ImageView::setImage(const QImage & image)
   setMinimumSize(std::min(640, image.width()), std::min(480, image.height()));
 }
 
-void ImageView::save(const QString & filename)
+bool ImageView::save(const QString & filename, int quality)
 {
-  _image.save(filename);
+  QString ext = QFileInfo(filename).suffix().toLower();
+  if ((ext == "jpg" || ext == "jpeg") && (quality == -1)) {
+    quality = JpegQualityDialog::ask(dynamic_cast<QWidget *>(parent()), -1);
+  }
+  if (quality == -1) {
+    return false;
+  }
+  if (!_image.save(filename, nullptr, quality)) {
+    QMessageBox::critical(this, tr("Error"), tr("Could not write image file %1").arg(filename));
+    return false;
+  }
+  return true;
 }
 
 ImageDialog::ImageDialog(QWidget * parent) : QDialog(parent)
 {
+  setWindowTitle(tr("G'MIC-Qt filter output"));
+  _jpegQuality = UNSPECIFIED_JPEG_QUALITY;
   auto vbox = new QVBoxLayout(this);
 
   _tabWidget = new QTabWidget(this);
@@ -56,10 +79,10 @@ ImageDialog::ImageDialog(QWidget * parent) : QDialog(parent)
 
   auto hbox = new QHBoxLayout;
   vbox->addLayout(hbox);
-  _closeButton = new QPushButton("Close");
+  _closeButton = new QPushButton(tr("Close"));
   connect(_closeButton, SIGNAL(clicked(bool)), this, SLOT(onCloseClicked(bool)));
   hbox->addWidget(_closeButton);
-  _saveButton = new QPushButton("Save as...");
+  _saveButton = new QPushButton(tr("Save as..."));
   connect(_saveButton, SIGNAL(clicked(bool)), this, SLOT(onSaveAs()));
   hbox->addWidget(_saveButton);
 }
@@ -68,8 +91,9 @@ void ImageDialog::addImage(const cimg_library::CImg<float> & image, const QStrin
 {
   auto view = new ImageView(_tabWidget);
   view->setImage(image);
-  _tabWidget->addTab(view, name);
+  _tabWidget->addTab(view, name + "*");
   _tabWidget->setCurrentIndex(_tabWidget->count() - 1);
+  _savedTab.push_back(false);
 }
 
 const QImage & ImageDialog::currentImage() const
@@ -85,20 +109,49 @@ int ImageDialog::currentImageIndex() const
   return _tabWidget->currentIndex();
 }
 
+void ImageDialog::supportedImageFormats(QStringList & extensions, QString & filters)
+{
+  extensions.clear();
+  for (const auto & ext : QImageWriter::supportedImageFormats()) {
+    extensions.push_back(QString::fromLatin1(ext).toLower());
+  }
+  QStringList filterList;
+  for (const auto & extension : extensions) {
+    QString filter = QString(tr("%1 file (*.%2)")).arg(extension.toUpper()).arg(extension);
+    if (extension == "png" || extension == "jpg" || extension == "jpeg") {
+      filterList.push_front(filter);
+    } else {
+      filterList.push_back(filter);
+    }
+  }
+  filters = filterList.join(";;");
+}
+
+void ImageDialog::setJPEGQuality(int q)
+{
+  _jpegQuality = q;
+}
+
 void ImageDialog::onSaveAs()
 {
   QString selectedFilter;
-  QString filename = QFileDialog::getSaveFileName(this, "Save image as...", QString(), "PNG file (*.png);;JPEG file (*.jpg)", &selectedFilter);
+  QStringList extensions;
+  QString filters;
+  supportedImageFormats(extensions, filters);
+  QString filename = QFileDialog::getSaveFileName(this, tr("Save image as..."), QString(), filters, &selectedFilter);
   QString extension = selectedFilter.split("*").back();
   extension.chop(1);
-  const QString upper = filename.toUpper();
-  if (!upper.endsWith(".PNG") && !upper.endsWith(".JPG") && !upper.endsWith(".JPEG")) {
+  if (!extensions.contains(QFileInfo(filename).suffix())) {
     filename += extension;
   }
   if (!filename.isEmpty()) {
     auto view = dynamic_cast<ImageView *>(_tabWidget->currentWidget());
+    int index = _tabWidget->currentIndex();
     if (view) {
-      view->save(filename);
+      if (view->save(filename, _jpegQuality)) {
+        _tabWidget->setTabText(index, QFileInfo(filename).fileName());
+        _tabWidget->setTabToolTip(index, QFileInfo(filename).filePath());
+      }
     }
   }
 }
@@ -112,7 +165,7 @@ void ImageView::paintEvent(QPaintEvent *)
 {
   QPainter p(this);
   QImage displayed;
-  if ((_image.width() / (float)_image.height()) > (width() / (float)height())) {
+  if ((static_cast<float>(_image.width()) / static_cast<float>(_image.height())) > (static_cast<float>(width()) / static_cast<float>(height()))) {
     displayed = _image.scaledToWidth(width());
     p.drawImage(0, (height() - displayed.height()) / 2, displayed);
   } else {
@@ -125,3 +178,5 @@ const QImage & ImageView::image() const
 {
   return _image;
 }
+
+} // namespace gmic_qt_standalone
